% This file is part of the Attempto Parsing Engine (APE).
% Copyright 2008-2010, Kaarel Kaljurand <kaljurand@gmail.com>.
%
% The Attempto Parsing Engine (APE) is free software: you can redistribute it and/or modify it
% under the terms of the GNU Lesser General Public License as published by the Free Software
% Foundation, either version 3 of the License, or (at your option) any later version.
%
% The Attempto Parsing Engine (APE) is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
% PURPOSE. See the GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU Lesser General Public License along with the Attempto
% Parsing Engine (APE). If not, see http://www.gnu.org/licenses/.


:- module(owlfss_ascii, [
		owlfss_ascii/2
	]).


/** <module> OWL 2 Functional-Style Syntax pretty-printer

@author Kaarel Kaljurand
@version 2010-07-05

*/


owlfss_ascii(OwlFss, OwlFssPp) :-
	with_output_to(atom(OwlFssPp), print_compound(0, OwlFss)).


print_compound(Level, OwlFss) :-
	OwlFss =.. [Name | Args],
	PrettyIndent is Level * 3,
	writef('%r%w(\n', [' ', PrettyIndent, Name]),
	NewLevel is Level + 1,
	print_list(NewLevel, Args),
	writef('%r)\n', [' ', PrettyIndent]).


print_list(_, []).

print_list(Level, [Head | Tail]) :-
	atomic(Head),
	!,
	PrettyIndent is Level * 3,
	writef('%r%w\n', [' ', PrettyIndent, Head]),
	print_list(Level, Tail).

print_list(Level, [Head | Tail]) :-
	is_list(Head),
	!,
	print_list(Level, Head),
	print_list(Level, Tail).

print_list(Level, [Expression | Tail]) :-
	pretty_print(Expression, PrettyExpression),
	!,
	PrettyIndent is Level * 3,
	writef('%r%w\n', [' ', PrettyIndent, PrettyExpression]),
	print_list(Level, Tail).

print_list(Level, [Head | Tail]) :-
	compound(Head),
	print_compound(Level, Head),
	print_list(Level, Tail).


% @deprecated
pretty_print('I-variable'('$VAR'(Number)), 'I-variable'(Number)) :- !.

pretty_print('I-variable'(Number), 'I-variable'(Number)).
pretty_print(Builtin, Builtin) :- functor(Builtin, builtIn, _).
pretty_print('AnonymousIndividual'('$VAR'(Number)), 'AnonymousIndividual'(Number)).
pretty_print('^^'(DataValue, DataType), '^^'(DataValue, DataType)).
pretty_print('InverseObjectProperty'(ObjectPropertyExpression), 'InverseObjectProperty'(ObjectPropertyExpression)).
pretty_print('ObjectProperty'(PropertyName), 'ObjectProperty'(PropertyName)).
pretty_print('DataProperty'(PropertyName), 'DataProperty'(PropertyName)).
pretty_print('Class'(ClassName), 'Class'(ClassName)).
pretty_print('Individual'(IndividualName), 'Individual'(IndividualName)).
